(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

(* module for assertions *)

exception Exc_assertion of string

(* an assertion consists of a name of a specification, a list ofpublic *)
(* symbols, an initial condition and a formula which is suppose to     *)
(* always hold.                                                        *)

type assertion =
    string * string list * Formulas.formula * Formulas.formula

(* (print_assertion flag oc assertion) prints assertion to the channel *)
(* oc, where flag indicates whether types should be printed.           *)
val print_assertion: bool -> out_channel -> assertion -> unit

(* check that an assertion satisfies the syntactic conditions, that is *)
(* that the two formulas are in the correct fragments of the logic     *)
(* raises excpetion Exc_assertion when this is not the case.            *)
val check_assertion_fragment : assertion -> unit

