(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

open Signatures
open Types
open Expressions
open Roles
open Declarations
open Axioms
open Scenarios 

type spec = signature * axioms * role list * sk_declaration list * scenario


(*************************************************************)
(*    Printing the abstract syntax                           *)
(*************************************************************)

let print_spec showtype oc
    (signature, (ax_decl,ax_eqs), rolelist, vars, scenario) =
  let os = output_string oc
  in
    Signatures.print_signature oc signature;

    os "axioms\n";
    os "declare\n";
    List.iter
      (function decl ->
	 os "  ";
	 os (string_of_declaration decl);
	 os ";\n")
      ax_decl;
    os "begin\n";
    List.iter
      (function (eq) ->
	 os "  ";
	 os (string_of_expression showtype eq);
	 os ";\n")
      ax_eqs;
    os "end\n\n";
    
    List.iter (print_role showtype oc) rolelist;
    
    os "\nvariables\n";
    List.iter 
      (function sk_decl ->
	 os "  ";
	 os (string_of_sk_declaration sk_decl);
	 os ";\n")
      vars;	
    os "end\n\n";
    
    os "\nscenario\n";
    print_scenario showtype oc scenario;
    os "\nend\n";
    
    close_out oc

